TITLE   KeySpeed        Author: Rick Housh
Page    60,120

Comment ~
  This program works only with AT and PS/2 compatible computers.
   It uses function 3 of ROM BIOS interrupt 16h to reset the
   keyboard initial delay and repeat rate (called "Typeamatic"
   by IBM).  There are six options available on the command line
   (in addition to help).  They are Extra Fast, which provides an
   initial delay of 250 milliseconds and a repeat rate of 30
   characters per second; Fast, 500 ms initial delay, 20 cps repeat
   rate; Medium, 500 ms initial delay, 15 cps repeat rate; Slow, 750 ms
   initial delay, 10 cps repeat rate; and Crawl, 1 second initial delay
   and 2 cps repeat rate.  The first letter of the speed is the parameter
   to supply on the command line, e.g. KEYSPEED /E will set the
   keyboard to Extra Fast.  Help is available with either /? or /H.
   
   The program will fail with an error message if its use on other
   than an AT or PS/2 compatible is attempted.  It may not work on
   some very early AT's.  This program is NOT a TSR, and therefore
   takes up no memory after being run.  You may reset the keyboard
   from the DOS command line whenever you wish.
   
   Placed in the public domain 5/1/1989.
   
   				Rick Housh
   				CIS PIN# 72466,212
				5811 W. 85th Terr.
				Overland Park, KS 66207
   
   End of comment, code follows ~

LF      EQU     0Ah                             ; Line feed equate
CR      EQU     0Dh                             ; Carriage return equate
;
;
_TEXT   SEGMENT
        ASSUME CS:_TEXT, DS:_TEXT, ES:_TEXT, SS:_TEXT

        ORG     100h                            ; For .COM file

Begin:  jmp     Start
;
FixType db      CR,9
Logo    db      CR,LF,'KeySpeed.Com by Rick Housh'
        db      CR,LF,'Enter KEYSPEED /? for help$'
HlpMsg  db      CR,'Sets keyboard character/sec repeat rate'
        db      CR,LF,'Syntax: KEYSPEED [Option]'
        db      CR,LF,'Options: /E - '
Extra   db      'Extra fast - (30 cps)'
        db      CR,LF,'         /F - '
Fast    db      'Fast ------- (20 cps)'
        db      CR,LF,'         /M - '
Med     db      'Medium ----- (15 cps)'
        db      CR,LF,'         /S - '
Slow    db      'Slow ------- (10 cps)'
        db      CR,LF,'         /C - '
Crawl   db      'Crawl ------ (02 cps)'
        db      CR,LF,'     /H, /? - '
        db      'Help ------- (No Change)$'
        db      8,' ',1Ah
DoneMsg db      ' is new repeat rate$'
ParmNG  db      CR,LF,7,'ERROR - Unknown Parameter '
ParmX   db      '  $'
ErrMsg  db      CR,LF,7,'ERROR - Unsupported computer'
        db      CR,LF,'Requires an AT or PS/2'
        db      CR,LF,' compatible machine$'

DisplayString Proc Near
        mov     ah,9
        int     21h
        ret
DisplayString EndP

Start:  push    es                              ; Save Extra Segment
        mov     ax,0F000h                       ; Get machine type from ROM
        mov     es,ax                           ; es gets machine ID segment
        mov     di,0FFFEh                       ; di gets machine ID offset
        mov     al,Byte Ptr es:[di]             ; Put ID byte in al
        pop     es                              ; Restore Extra Segment
        cbw                                     ; Make sure AH = 0
        cmp     al,0F9h                         ; PC Convertible? (Ugh)
        jz      NGMach                          ; Convertible No Good
        cmp     al,0FBh                         ; XT's are FEh or FBh
        jz      NGMach                          ; XT No Good, else
                                                ; If less than FDh (PCjr - Ugh)
                                                ; then is PS/2 Model 30, 50,
        cmp     al,0FDh                         ; 60, 70, 80, AT, XT286, and
        jb      OkMach                          ; is OK to proceed, else
						; fall through to error exit
NGMach: mov     dx,Offset ErrMsg                ; Display error message
        call    DisplayString                   ; then,
        mov     al,1                            ; exit with errorlevel 1
        jmp     ErrX

OkMach: mov     si,80h                          ; Point to CmdLine Length
        lodsb                                   ; Length to al
        cmp     al,0                            ; If something there, then
        jnz     RdParm                          ; process CmdLine,

NoParm: mov     dx,Offset Logo                  ; else give short help
        call    DisplayString
        jmp     NoErrX                          ; and exit without errorlevel
                                                ; else,
RdParm: cld                                     ; In forward direction
DoCdLn: lodsb                                   ; Load next Command Line byte
        cmp     al,CR                           ; If carriage return, finished
        jz      NoParm                          ; with no parms, so give help
        cmp     al,32                           ; If space
        jz      DoCdLn                          ; get another
        cmp     al,9                            ; Also tab
        jz      DoCdLn                          
        cmp     al,'-'                          ; Also hyphen
        jz      DoCdLn
        cmp     al,'/'                          ; Also slash
        jz      DoCdLn                          ; Otherwise,
        cmp     al,'?'                          ; if NOT question mark
        jnz     NotQMk                          ; then check it out

IsQMk:  mov     dx,Offset Logo                  ; Else display Logo
        call    DisplayString
        mov     dx,Offset HlpMsg                ; then long help
        call    DisplayString
        jmp     NoErrX                          ; and exit with no errorlevel   

NotQMk: cmp     al,'a'                          ; Lower than ASCII 'a'?
        jb      Chk4Up                          ; Yes? Then check parameters
        cmp     al,'z'                          ; Above lower case 'z'?
        ja      BadPrm                          ; Then can't be correct
        and     al,5fh                          ; Else make upper case
                                                ; and fall through to check for
                                                ; ASCII upper case characters
Chk4Up: cmp     al,'A'                          ; If lower than ASCII 'A'
        jb      BadPrm                          ; is NOT OK, so display error
        cmp     al,'Z'                          ; If higher than ASCII 'Z'
        ja      BadPrm                          ; is NOT OK, so display error,
                                                ; else,
Chk4_H: cmp     al,'H'                          ; if help requested
        jz      IsQMk                           ; then treat as '?'
                                                ; Else, may be good parameter
Chk4_E: cmp     al,'E'                          ; If NOT Extra Fast
        jnz     Chk4_F                          ; check Fast, else IS Extra,
        mov     al,'$'                          ; so poke in dollar sign
        mov     di,Offset Extra+21              ; at end of "Extra" message
        stosb
        mov     dx,Offset Extra                 ; and display it
        mov     bx,0000h                        ; Then set to 250ms delay,
                                                ; and 30 cps repeat
        jmp     short SetRpt                    ; and go set repeat rate

Chk4_F: cmp     al,'F'                          ; If NOT Fast
        jnz     Chk4_M                          ; check medium, else IS Fast,
        mov     al,'$'                          ; so poke in dollar sign
        mov     di,Offset Fast+21               ; at end of "Fast" message
        stosb
        mov     dx,Offset Fast                  ; and display it
        mov     bx,0104h                        ; Then set to 500ms delay,
                                                ; and 20 cps repeat
        jmp     short SetRpt                    ; and set repeat rate   

Chk4_M: cmp     al,'M'                          ; If NOT Medium
        jnz     Chk4_S                          ; Check slow, else IS Medium,
        mov     al,'$'                          ; so poke in dollar sign
        mov     di,Offset Med+21                ; at end of "Medium" message
        stosb
        mov     dx,Offset Med                   ; and display it
        mov     bx,0108h                        ; Then set to 500ms delay,
                                                ; and 15 cps repeat
        jmp     short SetRpt                    ; and set repeat rate

Chk4_S: cmp     al,'S'                          ; If NOT Slow
        jnz     Chk4_C                          ; Check crawl, else IS Slow
        mov     al,'$'                          ; so poke in dollar sign
        mov     di,Offset Slow+21               ; at end of "Slow" message
        stosb
        mov     dx,Offset Slow                  ; and display it
        mov     bx,020Ch                        ; Then set to 750ms delay,
                                                ; and 10 cps repeat
        jmp     short SetRpt                    ; and set repeat rate

Chk4_C: cmp     al,'C'                          ; If not Crawl
        jnz     PokPrm                          ; Then bad parameter, else
        mov     al,'$'                          ; poke in dollar sign
        mov     di,Offset Crawl+21              ; at end of "Crawl" message
        stosb
        mov     dx,Offset Crawl                 ; and display it
        mov     bx,031Fh                        ; Then set to 1 sec delay,
                                                ; and 2.0 cps repeat
        jmp     short SetRpt                    ; and set repeat rate

PokPrm: mov     di,Offset ParmX                 ; If the illegal parm is alpha
        mov     Byte Ptr es:[di],'/'            ; First add switch char to msg
        inc     di                              ; then, at next position,
        mov     Byte Ptr es:[di],al             ; add illegal character

BadPrm: mov     dx,Offset ParmNG                ; If NO legal parameter,
        call    DisplayString                   ; display bad Parm msg,
        mov     dx,Offset Logo                  ; then short help
        call    DisplayString
        mov     al,2                            ; Set errorlevel 2
        jmp     short ErrX                      ; and exit

SetRpt: push    dx                              ; Save Offset of message
        mov     ah,2                            ; Start with a line feed
        mov     dl,LF                           ; Using DOS
        int     21h                             ; to display
        pop     dx                              ; Restore Offset of Msg
        call    DisplayString                   ; and display first part
        mov     ax,0305h                        ; Set for ROM BIOS key repeat
        int     16h                             ; function, and set repeat rate
                                                ; This may stuff keyboard
KbLoop: mov     ax,0100h                        ; buffer, so check it with
        int     16h                             ; ROM kbd status function
        jz      NoKey                           ; If nothing there, proceed
        xor     ax,ax                           ; else,
        int     16h                             ; eat key
        jmp     short KBLoop                    ; and check for another

NoKey:  mov     dx,Offset DoneMsg               ; Say finished
        call    DisplayString

NoErrX: mov     al,0                            ; Set 0 errorlevel
ErrX:   push    ax                              ; Save errorlevel
        mov     ah,02                           ; Display
	mov	dl,CR				; One carriage return
	int	21h				;  and
        mov     dl,LF                           ; One linefeed
        int     21h                             ; Using DOS char. output
        pop     ax                              ; Restore errorlevel
        mov     ah,4Ch                          ; and exit
        int     21h     
        _TEXT   ENDS
;
END     Begin
